!  wsq_demo.f90 
!
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   wsq_demoSub()  - Callback routine for the main dialog box
!   wsq_demoOpen() - Callback routine for the "Open" button
!   wsq_demoSave() - Callback routine for the "Save" button
!   wsq_demoConverter() - Callback routine for the "Converter" button
!   wsq_demoRegister() - Callback routine for the "Register" button
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************

integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use dflogm
    use wsq_demoGlobals
	
    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'resource.fd'

    external wsq_demoSub
    external wsq_demoOpen
	external wsq_demoSave
	external wsq_demoRegister


    ! Variables
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret

	ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL

    lret = DlgInit(IDD_WSQ_DEMO_DIALOG, gdlg)

    if (lret == .FALSE.) goto 99999

    lret = DlgSetSub(gdlg, IDD_WSQ_DEMO_DIALOG, wsq_demoSub)
    lret = DlgSetSub(gdlg, IDC_OPEN_BUTTON, wsq_demoOpen)
	lret = DlgSetSub(gdlg, IDC_SAVE_BUTTON, wsq_demoSave)
	lret = DlgSetSub(gdlg, IDC_REGISTER_BUTTON, wsq_demoRegister)


	lret = DlgModal ( gdlg) 
    call DlgUninit(gdlg)
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application wsq_demo"C, &
                     "Error"C, MB_OK)
    WinMain = 0

end

SUBROUTINE DoDialog( )
    USE DFLOGM
    IMPLICIT NONE
    INCLUDE 'RESOURCE.FD'

    INTEGER retint
    LOGICAL retlog
    TYPE (dialog) dlg
    
  ! Initialize.
    IF ( .not. DlgInit( IDD_WSQ_DEMO_DIALOG, dlg ) ) THEN
        WRITE (*,*) "Error: dialog not found"
    ELSE

        retint = DlgModal( dlg )

        CALL DlgUninit( dlg )
    END IF
END SUBROUTINE DoDialog


!****************************************************************************
!
!  FUNCTION: wsq_demoSub ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for initialization and destroy
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE wsq_demoSub( dlg, id, callbacktype )
!DEC$ ATTRIBUTES DEFAULT :: wsq_demoSub


  use user32
  use dflogm

  use dfwin 
  use wsq_demoGlobals 



  implicit none


  type (dialog) dlg
  integer id, callbacktype
  integer ret 


!Locate the WSQ image library dll and load it into memory
	hWSQLib = loadlibrary("WSQ_library_fortranexport.dll"C)
	if (hWSQLib == 0) then
		ret = MessageBox (0, & 
             "Error: file WSQ_library_fortranexport.dll not found."// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
		call PostQuitMessage(0)	
	endif

! Set up a pointers to the routines of interest
	pCreateBMPFromFile = getprocaddress(hWSQLib, "_CreateBMPFromFile"C)
	pSaveBMPToFile = getprocaddress(hWSQLib, "_SaveBMPToFile"C)
	pRegisterWSQ = getprocaddress(hWSQLib, "_RegisterWSQ"C)
	pSetShowFilePropertiesDialog = getprocaddress(hWSQLib, "_SetShowFilePropertiesDialog"C)
	pShowFileConverter = getprocaddress(hWSQLib, "_ShowFileConverter"C)



	if (pCreateBMPFromFile == 0) then
       ret = MessageBox (0, & 
             "Error: Function CreateBMPFromFile is not found in file WSQ_library_fortranexport.dll"// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
	   call PostQuitMessage(0)
	endif


	if (pSaveBMPToFile == 0) then
       ret = MessageBox (0, & 
             "Error: Function SaveBMPToFile is not found in file WSQ_library_fortranexport.dll"// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
	   call PostQuitMessage(0)
	endif


	if (pRegisterWSQ == 0) then
       ret = MessageBox (0, & 
             "Error: Function RegisterWSQ is not found in file WSQ_library_fortranexport.dll"// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
	   call PostQuitMessage(0)
	endif



	if (pSetShowFilePropertiesDialog == 0) then
       ret = MessageBox (0, & 
             "Error: Function SetShowFilePropertiesDialog is not found in file WSQ_library_fortranexport.dll"// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
	   call PostQuitMessage(0)
	endif


	if (pShowFileConverter == 0) then
       ret = MessageBox (0, & 
             "Error: Function ShowFileConverter is not found in file WSQ_library_fortranexport.dll"// CHAR(13)// & 
             "Press OK to quit."C, & 
             "WSQ demo"C,MB_OK+MB_ICONERROR) 
	   call PostQuitMessage(0)
	endif




  if (callbacktype == dlg_destroy) then

	status = freelibrary(hWSQLib) !Unload WSQ image library DLL  

	call PostQuitMessage(0)
  endif

  END SUBROUTINE wsq_demoSub

!****************************************************************************
!
!  FUNCTION: wsq_demoOpen ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for "Open" button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE wsq_demoOpen( dlg, id, callbacktype )
!DEC$ ATTRIBUTES DEFAULT :: wsq_demoOpen


    use dfwin
    use dflogm
    use dfcom
    use dfauto
	use msfwin

    use wsq_demoGlobals 

    implicit none

    type (dialog) dlg
    integer id, callbacktype

    include 'resource.fd'

    type(T_OPENFILENAME) fileinfo
    character * MAX_PATH filename
    character * 2048 filefilter
    integer obj, iret
    logical lret

	integer hb
	integer iret2
	integer(HANDLE) hStatic
	
	integer hdcSrc
    integer hdcDst
    type (T_BITMAP)  bm

	integer*4 newBitmap, oldBitmap, oldBitmap2
	integer i
	integer width
	integer height

    if (callbacktype == dlg_clicked) then


   !Enable "Save" Button
   lret = DLGSET (dlg, IDC_SAVE_BUTTON, .TRUE., DLG_ENABLE)


        ! Display a file open dialog box to allow the user to choose the
        ! multimedia file to play
        fileinfo % lStructSize = sizeof(fileinfo)
        fileinfo % hwndOwner = dlg % hwnd
        fileinfo % hInstance = 0


		iret = lstrcpy(filefilter, &
"All images|*.wsq;*.bmp;*.tiff;*.tif;*.jpg;*.jp2;*.jpc;*.jpe;*.jpeg;*.png;*.gif;*.iff;*.ico;*.wmf;*.emf;*.jfif;*.rle;*.dib;*.win;*.vst;*.vda;*.tga; &
*.icb;*.fax;*.eps;*.pcx;*.pcc;*.scr; *.rpf;*.rla;*.sgi;*.rgba;*.rgb;*.bw;*.psd;*.pdd;*.ppm;*.pgm;*.pbm;*.cel;*.pic;*.pcd;*.cut;*.psp;| &
FBI Wavelet Scalar Quantization (*.wsq)|*.wsq|Windows bitmaps (*.bmp)|*.bmp|Macintosh TIFF images (*.tiff)|*.tiff|PC TIFF images (*.tif)|*.tif|JPG images (*.jpg)|*.jpg| &
JPEG-2000 Part-1 (*.jp2)|*.jp2|JPEG-2000 Code Stream (*.jpc)|*.jpc|JPE images (*.jpe)|*.jpe|JPEG images (*.jpeg)|*.jpeg|Portable network graphic images (*.png)|*.png| &
Compuserve Graphic Interchange Format(*.gif)|*gif|Paintshop Pro images (*.psp)|*.psp|JFIF images (*.jfif)|*.jfif| &
Run length encoded Windows bitmaps (*.rle)|*.rle|Device independant Windows bitmaps (*.dib)|*.dib|Truevision images (*.win)|*.win| &
Truevision images (*.vst)|*.vst|Truevision images (*.vda)|*.vda|Truevision images (*.tga)|*.tga|Truevision images (*.icb)|*.icb| &
GFI fax images (*.fax)|*.fax|Encapsulated Postscript images (*.eps)|*.eps|ZSoft Paintbrush images (*.pcx)|*.pcx| &
ZSoft Paintbrush images (*.pcc)|*.pcc|Word 5.x screen capture images (*.scr)|*.scr|Alias/Wavefront images (*.rpf)|*.rpf| &
Alias/Wavefront images (*.rla)|*.rla|SGI true color images (*.sgi)|*.sgi|SGI true color images with alpha (*.rgba)|*.rgba| &
SGI true color images (*.rgb)|*.rgb|SGI black/white images (*.bw)|*.bw|Photoshop images (*.psd)|*.psd|Photoshop images (*.pdd)|*.pdd| &
Portable pixel map images (*.ppm)|*.ppm|Portable gray map images (*.pgm)|*.pgm|Portable bitmap images (*.pbm)|*.pbm| &
Autodesk images (*.cel)|*.cel|Autodesk images (*.pic)|*.pic|Kodak Photo-CD images (*.pcd)|*.pcd|Dr. Halo images (*.cut)|*.cut| &
Amiga Bitmap Graphic (8 bits)|*.iff|Windows icons (*.ico)|*.ico|Windows metafiles (*.wmf)|*.wmf|Windows enhanced meta files (*.emf)|*.emf||"C)



		call convertfilterstring(filefilter)
        fileinfo % lpstrFilter = loc(filefilter)
        fileinfo % lpstrCustomFilter = 0
        fileinfo % nMaxCustFilter = 0
        fileinfo % nFilterIndex = 0
        filename = ""C
        fileinfo % lpstrFile = loc(filename)
        fileinfo % nMaxFile = MAX_PATH
        fileinfo % lpstrFileTitle = 0
        fileinfo % nMaxFileTitle = 0
        fileinfo % lpstrInitialDir = 0
        fileinfo % lpstrTitle = 0
        fileinfo % Flags = OFN_HIDEREADONLY
        fileinfo % nFileOffset = 0
        fileinfo % nFileExtension = 0
        fileinfo % lpstrDefExt = 0
        fileinfo % lCustData = 0
        fileinfo % lpfnHook = 0
        fileinfo % lpTemplateName = 0

        lret = GetOpenFileName(fileinfo)
        if (lret .ne. 0) then

			hb = CreateBMPFromFile(filename)

	

	hdcSrc = CreateCompatibleDC(0)
	hdcDst = CreateCompatibleDC(0)

	i = GetObject (hb, 24, LOC(bm))
	oldBitmap = SelectObject(hdcSrc, hb)

	width = ZEXT(bm % bmWidth)
	height = ZEXT(bm % bmHeight)	

	baseBitmap = CreateBitmap( width, height, 1, 32, 0 );

	oldBitmap2 = SelectObject(hdcDst, baseBitmap)

	i = BitBlt(hdcDst, 0, 0, width, height, hdcSrc, 0, 0, SRCCOPY);

	i = SelectObject(hdcSrc, oldBitmap)
    i = DeleteDC(hdcSrc);
    i = DeleteDC(hdcDst);

			hStatic = GetDlgItem(dlg%hWnd, IDC_OUTPUT_IMAGE)
			iret2 = SendMessage(hStatic, STM_SETIMAGE, IMAGE_BITMAP, hb)
        endif
    endif

END SUBROUTINE wsq_demoOpen

SUBROUTINE saveBmp (bitmap, filename, filetype)

END SUBROUTINE

SUBROUTINE wsq_demoSave( dlg, id, callbacktype )
!DEC$ ATTRIBUTES DEFAULT :: wsq_demoSave

    use dfwin
    use dflogm
    use dfcom
    use dfauto

    use wsq_demoGlobals 
	

    implicit none

    type (dialog) dlg
    integer id, callbacktype

    include 'resource.fd'

    type(T_OPENFILENAME) fileinfo
    character * MAX_PATH filename
    character * 2048 filefilter

    integer obj, iret
    logical lret

	integer(HANDLE) hStatic
	integer(HANDLE) hStatic2
	integer*4 hBitmap;
	integer ret
	integer iret2

	integer hdcSrc
    integer hdcDst
    type (T_BITMAP)  bm

	integer*4 newBitmap, oldBitmap, oldBitmap2
	integer i, width, height
	integer*4 filterIndex

	character*20, POINTER :: pn

    if (callbacktype == dlg_clicked) then

        ! Display a file save dialog box to allow the user to choose the
        ! multimedia file to play
        fileinfo % lStructSize = sizeof(fileinfo)
        fileinfo % hwndOwner = dlg % hwnd
        fileinfo % hInstance = 0
		iret = lstrcpy(filefilter, &
"WSQ Files (*.wsq)|*.wsq|&
Windows BMP (*.bmp)|*.bmp|&
TIFF Files (*.tif)|*.tif|&
PNG Files (*.png)|*.png|&
JPEG Files (*.jpg)|*.jpg|&
Targa Files (*.tga)|*.tga|&
SGI Files (*.rgb)|*.rgb|&
JPEG-2000 Part-1 (*.jp2)|*.jp2|&
JPEG-2000 Code Stream (*.jpc)|*.jpc|&
PBM Files (*.pbm)|*.pbm|&
PGM Files (*.pgm)|*.pgm|&
PPM Files (*.ppm)|*.ppm||"C)


		call convertfilterstring(filefilter)
        fileinfo % lpstrFilter = loc(filefilter)
        fileinfo % lpstrCustomFilter = 0
        fileinfo % nMaxCustFilter = 0
        fileinfo % nFilterIndex = 0
        filename = ""C
        fileinfo % lpstrFile = loc(filename)
        fileinfo % nMaxFile = MAX_PATH
        fileinfo % lpstrFileTitle = 0
        fileinfo % nMaxFileTitle = 0
        fileinfo % lpstrInitialDir = 0
        fileinfo % lpstrTitle = 0
        fileinfo % Flags = OFN_HIDEREADONLY
        fileinfo % nFileOffset = 0
        fileinfo % nFileExtension = 0
        fileinfo % lpstrDefExt = 0
        fileinfo % lCustData = 0
        fileinfo % lpfnHook = 0
        fileinfo % lpTemplateName = 0

        lret = GetSaveFileName(fileinfo)
        if (lret .ne. 0) then

	hdcSrc = CreateCompatibleDC(0)
	hdcDst = CreateCompatibleDC(0)

	i = GetObject (baseBitmap, 24, LOC(bm))
	oldBitmap = SelectObject(hdcSrc, baseBitmap)

	width = ZEXT(bm % bmWidth)
	height = ZEXT(bm % bmHeight)

	hBitmap = CreateBitmap( width, height, 1, 32, 0 );

	oldBitmap2 = SelectObject(hdcDst, hBitmap)

	i = BitBlt(hdcDst, 0, 0, width, height, hdcSrc, 0, 0, SRCCOPY);

	i = SelectObject(hdcSrc, oldBitmap)
    i = DeleteDC(hdcSrc);
    i = DeleteDC(hdcDst);

			filterIndex = fileinfo % nFilterIndex
            i = SaveBMPToFile(%VAL(hBitmap), filename, %VAL(filterIndex))
        endif
    endif

END SUBROUTINE wsq_demoSave




SUBROUTINE wsq_demoRegister( dlg, id, callbacktype )
!DEC$ ATTRIBUTES DEFAULT :: wsq_demoRegister

  use dflogm
  use wsq_demoGlobals 


  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_clicked) then
   call RegisterWSQ
  endif

  END SUBROUTINE wsq_demoRegister
